@@ -1,6 +1,6 @@
 # This script generates the PyQt configuration and generates the Makefiles.
 #
-# Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
+# Copyright (c) 2013 Riverbank Computing Limited <info@riverbankcomputing.com>
 # 
 # This file is part of PyQt.
 # 
@@ -33,10 +33,10 @@ import sipconfig
 
 
 # Initialise the globals.
-pyqt_version = 0x040904
-pyqt_version_str = "4.9.4"
+pyqt_version = 0x040a02
+pyqt_version_str = "4.10.2"
 
-sip_min_version = 0x040d03
+sip_min_version = 0x040e03
 
 qt_version = 0
 qt_edition = ""
@@ -46,6 +46,7 @@ qt_incdir = None
 qt_libdir = None
 qt_bindir = None
 qt_datadir = None
+qt_archdatadir = None
 qt_pluginsdir = None
 qt_xfeatures = None
 qt_shared = ""
@@ -74,9 +75,9 @@ dbuslibs = []
 # dynamic linker won't be able to resolve the symbols.  On other systems we
 # assume we can just run qmake by using its full pathname.
 if sys.platform == 'win32':
-    MSG_CHECK_QMAKE = "Make sure you have a working Qt v4 qmake on your PATH."
+    MSG_CHECK_QMAKE = "Make sure you have a working Qt qmake on your PATH."
 else:
-    MSG_CHECK_QMAKE = "Make sure you have a working Qt v4 qmake on your PATH or use the -q argument to explicitly specify a working Qt v4 qmake."
+    MSG_CHECK_QMAKE = "Make sure you have a working Qt qmake on your PATH or use the -q argument to explicitly specify a working Qt qmake."
 
 
 def find_default_qmake():
@@ -177,6 +178,9 @@ def create_optparser():
     g.add_option("-T", "--no-timestamp", action="store_true", default=False,
             dest="no_timestamp", help="suppress timestamps in the header "
             "comments of generated code [default: include timestamps]")
+    g.add_option("--no-deprecated", action="store_false", default=True,
+            dest="with_deprecated", help="disable Qt v4 features deprecated "
+            "in Qt v5 [default: enabled]")
 
     if sys.platform != 'win32':
         if sys.platform.startswith('linux') or sys.platform == 'darwin':
@@ -273,8 +277,8 @@ def create_optparser():
 
 
 class pyrccMakefile(sipconfig.ProgramMakefile):
-    """This class implements the Makefile for pyrcc.  This is specialised so
-    that pyrcc is automatically run against the examples.
+    """This class implements the Makefile for pyrcc.  This was specialised so
+    that pyrcc was automatically run against the examples (no longer done).
     """
 
     def __init__(self):
@@ -285,28 +289,6 @@ class pyrccMakefile(sipconfig.ProgramMakefile):
                 universal=sipcfg.universal, arch=sipcfg.arch,
                 deployment_target=sipcfg.deployment_target)
 
-    def generate_target_default(self, mfile):
-        """Generate the default target."""
-        sipconfig.ProgramMakefile.generate_target_default(self, mfile)
-
-        # The correct call to pyrcc depends on the Python version.
-        if sys.hexversion >= 0x03000000:
-            flag = "-py3"
-        else:
-            flag = "-py2"
-
-        exe = "$(TARGET)"
-        if sys.platform != 'win32':
-            exe = "./" + exe
-
-        # Find all the .qrc files in the examples.
-        for root, _, files in os.walk("examples"):
-            rel_root = os.path.join("..", root)
-
-            for fn in files:
-                if fn.endswith(".qrc"):
-                    mfile.write("\t%s %s -o %s %s\n" % (exe, flag, os.path.join(rel_root, fn[:-4] + "_rc.py"), os.path.join(rel_root, fn)))
-
 
 class ConfigurePyQt4:
     """This class defines the methods to configure PyQt4.
@@ -339,7 +321,9 @@ class ConfigurePyQt4:
             0x040701: "Qt_4_7_0",
             0x040702: "Qt_4_7_1",
             0x040800: "Qt_4_7_2",
-            0x050000: "Qt_4_8_0",
+            0x040803: "Qt_4_8_0",
+            0x040804: "Qt_4_8_3",
+            0x050000: "Qt_4_8_4",
             0x060000: "Qt_5_0_0"
         }
 
@@ -361,41 +345,43 @@ class ConfigurePyQt4:
         pyqt_modules.append("QtCore")
 
         check_module("QtGui", "qwidget.h", "new QWidget()")
-        if qt_version < 0x050000: check_module("QtHelp", "qhelpengine.h", "new QHelpEngine(\"foo\")")
-        if qt_version < 0x050000: check_module("QtMultimedia", "QAudioDeviceInfo",
+        check_module("QtHelp", "qhelpengine.h", "new QHelpEngine(\"foo\")")
+        check_module("QtMultimedia", "QAudioDeviceInfo",
                 "new QAudioDeviceInfo()")
-        if qt_version < 0x050000: check_module("QtNetwork", "qhostaddress.h", "new QHostAddress()")
-        if qt_version < 0x050000: check_module("QtDBus", "qdbusconnection.h",
-                "QDBusConnection::systemBus()")
-        if qt_version < 0x050000: check_module("QtDeclarative", "qdeclarativeview.h",
+        check_module("QtNetwork", "qhostaddress.h", "new QHostAddress()")
+
+        # Qt v4.7 was current when we added support for QtDBus and we didn't
+        # bother properly versioning its API.
+        if qt_version >= 0x040700:
+            check_module("QtDBus", "qdbusconnection.h",
+                    "QDBusConnection::systemBus()")
+
+        check_module("QtDeclarative", "qdeclarativeview.h",
                 "new QDeclarativeView()")
-        if qt_version < 0x050000: check_module("QtOpenGL", "qgl.h", "new QGLWidget()")
-        if qt_version < 0x050000: check_module("QtScript", "qscriptengine.h", "new QScriptEngine()")
-        if qt_version < 0x050000: check_module("QtScriptTools", "qscriptenginedebugger.h",
+        check_module("QtOpenGL", "qgl.h", "new QGLWidget()")
+        check_module("QtScript", "qscriptengine.h", "new QScriptEngine()")
+        check_module("QtScriptTools", "qscriptenginedebugger.h",
                 "new QScriptEngineDebugger()")
-        if qt_version < 0x050000: check_module("QtSql", "qsqldatabase.h", "new QSqlDatabase()",
+        check_module("QtSql", "qsqldatabase.h", "new QSqlDatabase()",
                 extra_libs=sql_libs)
-        if qt_version < 0x050000: check_module("QtSvg", "qsvgwidget.h", "new QSvgWidget()")
-        if qt_version < 0x050000: check_module("QtTest", "QtTest", "QTest::qSleep(0)")
-        if qt_version < 0x050000: check_module("QtWebKit", "qwebpage.h", "new QWebPage()")
-        if qt_version < 0x050000: check_module("QtXml", "qdom.h", "new QDomDocument()")
-        if qt_version < 0x050000: check_module("QtXmlPatterns", "qxmlname.h", "new QXmlName()")
-        if qt_version < 0x050000: check_module("phonon", "phonon/videowidget.h",
+        check_module("QtSvg", "qsvgwidget.h", "new QSvgWidget()")
+        check_module("QtTest", "QtTest", "QTest::qSleep(0)")
+        check_module("QtWebKit", "qwebpage.h", "new QWebPage()")
+        check_module("QtXml", "qdom.h", "new QDomDocument()")
+        check_module("QtXmlPatterns", "qxmlname.h", "new QXmlName()")
+        check_module("phonon", "phonon/videowidget.h",
                 "new Phonon::VideoWidget()")
-        if qt_version < 0x050000: check_module("QtAssistant", "qassistantclient.h",
+        check_module("QtAssistant", "qassistantclient.h",
                 "new QAssistantClient(\"foo\")", extra_lib_dirs=ass_lib_dirs,
                 extra_libs=ass_libs)
 
         if not qt_shared:
             sipconfig.inform("QtDesigner module disabled with static Qt libraries.")
-        elif sipcfg.universal:
-            sipconfig.inform("QtDesigner module disabled with universal binaries.")
         else:
-            if qt_version < 0x050000: check_module("QtDesigner", "QExtensionFactory",
+            check_module("QtDesigner", "QExtensionFactory",
                     "new QExtensionFactory()")
 
-        if qt_version < 0x050000: check_module("QAxContainer", "qaxobject.h", "new QAxObject()",
-                extra_libs=["QAxContainer"])
+        check_module("QAxContainer", "qaxobject.h", "new QAxObject()")
 
         if os.path.isdir(os.path.join(src_dir, "dbus")):
             check_dbus()
@@ -669,22 +655,19 @@ class ConfigurePyQt4:
         for the given module.
 
         mname is the name of the module.
-        lib_name is the normal name of the support library.
+        lib_name is the normal name of the support library.  (Note that this is
+        historical and the unchanged name is always returned.)
         """
         qpy_dir = os.path.join("qpy", mname)
 
         if sys.platform == 'win32':
             if opts.debug:
                 qpy_lib_dir = os.path.join(qpy_dir, "debug")
-                lib_name = 'd' + lib_name
             else:
                 qpy_lib_dir = os.path.join(qpy_dir, "release")
         else:
             qpy_lib_dir = qpy_dir
 
-            if sys.platform == 'darwin' and opts.debug:
-                lib_name += '_debug'
-
         return os.path.join(src_dir, qpy_dir), os.path.abspath(qpy_lib_dir), lib_name
 
     def _static_plugins(self, mname):
@@ -809,6 +792,9 @@ class ConfigurePyQt4:
                 if sipcfg.sip_inc_dir != sipcfg.py_inc_dir:
                     inc_path.insert(0, sipcfg.sip_inc_dir)
 
+                if sipcfg.py_conf_inc_dir != sipcfg.py_inc_dir:
+                    inc_path.insert(0, sipcfg.py_conf_inc_dir)
+
                 if opts.bigqt:
                     api_dir = "../../_qt"
                 else:
@@ -926,6 +912,8 @@ include(%s)
         if "QtXml" in pyqt_modules:
             sipconfig.inform("Creating pylupdate4 Makefile...")
 
+            cxxflags_app = sipcfg.build_macros().get("CXXFLAGS_APP", "")
+
             makefile = sipconfig.ProgramMakefile(
                 configuration=sipcfg,
                 build_file=os.path.join(src_dir, "pylupdate", "pylupdate.sbf"),
@@ -940,13 +928,22 @@ include(%s)
                 deployment_target=sipcfg.deployment_target
             )
 
-            makefile.extra_include_dirs.append(os.path.join(src_dir, "pylupdate"))
+            makefile.extra_include_dirs.append(
+                    os.path.join(src_dir, "pylupdate"))
+
+            if cxxflags_app != "":
+                makefile.extra_cxxflags.append(cxxflags_app)
+
             makefile.generate()
             tool.append("pylupdate")
 
             sipconfig.inform("Creating pyrcc4 Makefile...")
 
             makefile = pyrccMakefile()
+
+            if cxxflags_app != "":
+                makefile.extra_cxxflags.append(cxxflags_app)
+
             makefile.generate()
             tool.append("pyrcc")
         else:
@@ -1090,10 +1087,16 @@ def inform_user():
     sipconfig.inform("The Qt header files are in %s." % qt_incdir)
     sipconfig.inform("The %s Qt libraries are in %s." % (lib_type, qt_libdir))
     sipconfig.inform("The Qt binaries are in %s." % qt_bindir)
-    sipconfig.inform("The Qt mkspecs directory is in %s." % qt_datadir)
+    sipconfig.inform("The Qt mkspecs directory is in %s." % qt_archdatadir)
     sipconfig.inform("These PyQt modules will be built: %s." % ", ".join(pyqt_modules))
     sipconfig.inform("The PyQt Python package will be installed in %s." % opts.pyqtmoddir)
 
+    if qt_version >= 0x050000:
+        if opts.with_deprecated:
+            sipconfig.inform("PyQt is being built with deprecated Qt v4 features.")
+        else:
+            sipconfig.inform("PyQt is being built without deprecated Qt v4 features.")
+
     if opts.no_docstrings:
         sipconfig.inform("PyQt is being built without generated docstrings.")
     else:
@@ -1145,6 +1148,7 @@ def create_config(module, template, macros):
         "qt_threaded":        1,
         "qt_dir":             qt_dir,
         "qt_data_dir":        qt_datadir,
+        "qt_archdata_dir":    qt_archdatadir,
         "qt_inc_dir":         qt_incdir,
         "qt_lib_dir":         qt_libdir
     }
@@ -1491,11 +1495,17 @@ def set_sip_flags(pyqt):
 
     pyqt is the configuration instance.
     """
+    sip_flags = []
+
     # If we don't check for signed interpreters, we exclude the 'VendorID'
     # feature
     if not opts.vendorcheck:
-        qt_sip_flags.append("-x")
-        qt_sip_flags.append("VendorID")
+        sip_flags.append("-x")
+        sip_flags.append("VendorID")
+
+    if qt_version >= 0x050000 and not opts.with_deprecated:
+        sip_flags.append("-x")
+        sip_flags.append("PyQt_Deprecated_5_0")
 
     # Handle the platform tag.
     if sys.platform == 'win32':
@@ -1508,35 +1518,40 @@ def set_sip_flags(pyqt):
     else:
         plattag = "WS_X11"
 
-    qt_sip_flags.append("-t")
-    qt_sip_flags.append(plattag)
+    sip_flags.append("-t")
+    sip_flags.append(plattag)
 
     # Handle the Qt version tag.
     verstag = sipconfig.version_to_sip_tag(qt_version, pyqt.qt_version_tags(), "Qt")
 
     # Handle any feature flags.
     for xf in qt_xfeatures:
-        qt_sip_flags.append("-x")
-        qt_sip_flags.append(xf)
+        sip_flags.append("-x")
+        sip_flags.append(xf)
 
     if verstag:
-        qt_sip_flags.append("-t")
-        qt_sip_flags.append(verstag)
+        sip_flags.append("-t")
+        sip_flags.append(verstag)
 
     # Handle the version specific Python features.
     if sipcfg.py_version < 0x020400:
-        qt_sip_flags.append("-x")
-        qt_sip_flags.append("Py_DateTime")
+        sip_flags.append("-x")
+        sip_flags.append("Py_DateTime")
 
     if sipcfg.py_version < 0x030000:
-        qt_sip_flags.append("-x")
-        qt_sip_flags.append("Py_v3")
+        sip_flags.append("-x")
+        sip_flags.append("Py_v3")
+
+    qt_sip_flags.extend(sip_flags)
 
     # There is an issue creating QObjects while the GIL is held causing
     # deadlocks in multi-threaded applications.  We don't fully understand this
     # yet so we make sure we avoid the problem by always releasing the GIL.
     qt_sip_flags.append("-g")
 
+    # Return the sip flags that sub-modules need to know about.
+    return " ".join(sip_flags)
+
 
 def needed_qt_libs(mname, qt_libs):
     """Add any additional Qt libraries needed by a module to an existing list.
@@ -1739,6 +1754,10 @@ def generate_code(mname, extra_include_dirs=None, extra_lib_dirs=None, extra_lib
 
     add_makefile_extras(makefile, extra_include_dirs, extra_lib_dirs, extra_libs)
 
+    if qt_version >= 0x050000 and opts.with_deprecated:
+        # The name of this macro is very confusing.
+        makefile.extra_defines.append('QT_DISABLE_DEPRECATED_BEFORE=0x040900')
+
     makefile.generate()
 
 
@@ -1871,19 +1890,29 @@ def get_build_macros(overrides):
 
         if not os.path.dirname(fname):
             qt_macx_spec = fname
-            fname = os.path.join(qt_datadir, "mkspecs", fname)
+            fname = os.path.join(qt_archdatadir, "mkspecs", fname)
     elif sys.platform == "darwin":
         # The Qt Mac binary installer defaults to xcode which we don't want.
         # Use Qt5's macx-clang if it is available, otherwise fall back to
         # macx-g++.
-        fname = os.path.join(qt_datadir, "mkspecs", "macx-clang")
+        fname = os.path.join(qt_archdatadir, "mkspecs", "macx-clang")
         if os.path.isdir(fname):
             qt_macx_spec = "macx-clang"
         else:
-            fname = os.path.join(qt_datadir, "mkspecs", "macx-g++")
+            fname = os.path.join(qt_archdatadir, "mkspecs", "macx-g++")
             qt_macx_spec = "macx-g++"
     else:
-        fname = os.path.join(qt_datadir, "mkspecs", "default")
+        fname = os.path.join(qt_archdatadir, "mkspecs", "default")
+
+        # Qt5 doesn't have the 'default' link.
+        if not os.path.isdir(fname):
+            f = get_command_stdout(opts.qmake + " -query QMAKE_SPEC")
+            fname = f.read().strip()
+            if sys.hexversion >= 0x03000000:
+                fname = fname.decode()
+            f.close()
+
+            fname = os.path.join(qt_archdatadir, "mkspecs", fname)
 
     fname = os.path.join(fname, "qmake.conf")
 
@@ -1907,7 +1936,14 @@ def get_build_macros(overrides):
     if macros is None:
         return None
 
-    # Make sure we have an entry for MOC (which Qt v5 doesn't provide).
+    # Qt5 doesn't seem to support the specific macros so add them if they are
+    # missing.
+    if macros.get("INCDIR_QT", "") == "":
+        macros["INCDIR_QT"] = qt_incdir
+
+    if macros.get("LIBDIR_QT", "") == "":
+        macros["LIBDIR_QT"] = qt_libdir
+
     if macros.get("MOC", "") == "":
         default_moc = os.path.join(qt_bindir, "moc")
         if sys.platform == 'win32':
@@ -1971,7 +2007,8 @@ def fix_qmake_args(args=""):
 
 def get_qt_configuration():
     """Set the qt_dir, qt_incdir, qt_libdir, qt_bindir, qt_datadir,
-    qt_pluginsdir and qt_xfeatures globals for the Qt installation.
+    qt_archdatadir, qt_pluginsdir and qt_xfeatures globals for the Qt
+    installation.
     """
     sipconfig.inform("Determining the layout of your Qt installation...")
 
@@ -2022,7 +2059,10 @@ SOURCES = %s
 #include <QLibraryInfo>
 #include <QTextStream>
 
-// This seems to be missing from the Qt v5 alpha.
+// These seem to be missing from the Qt v5 beta.
+#if !defined(QT_EDITION_DESKTOP)
+#define QT_EDITION_DESKTOP      8
+#endif
 #if !defined(QT_EDITION_OPENSOURCE)
 #define QT_EDITION_OPENSOURCE   8
 #endif
@@ -2042,6 +2082,11 @@ int main(int argc, char **argv)
     out << QLibraryInfo::location(QLibraryInfo::LibrariesPath) << '\\n';
     out << QLibraryInfo::location(QLibraryInfo::BinariesPath) << '\\n';
     out << QLibraryInfo::location(QLibraryInfo::DataPath) << '\\n';
+#if QT_VERSION >= 0x050000
+    out << QLibraryInfo::location(QLibraryInfo::ArchDataPath) << '\\n';
+#else
+    out << QLibraryInfo::location(QLibraryInfo::DataPath) << '\\n';
+#endif
     out << QLibraryInfo::location(QLibraryInfo::PluginsPath) << '\\n';
 
     out << QT_VERSION << '\\n';
@@ -2105,6 +2150,10 @@ int main(int argc, char **argv)
     out << "PyQt_PrintPreviewWidget\\n";
 #endif
 
+#if defined(QT_NO_RAWFONT)
+    out << "PyQt_RawFont\\n";
+#endif
+
 #if !defined(QT3_SUPPORT) || QT_VERSION >= 0x040200
     out << "PyQt_NoPrintRangeBug\\n";
 #endif
@@ -2151,14 +2200,15 @@ int main(int argc, char **argv)
     run_command(exe_file)
 
     if not os.access(out_file, os.F_OK):
-        sipconfig.error("%s failed to create %s. Make sure your Qt v4 installation is correct." % (exe_file, out_file))
+        sipconfig.error("%s failed to create %s. Make sure your Qt installation is correct." % (exe_file, out_file))
 
     # Read the directories.
     f = open(out_file, "r")
     lines = f.read().strip().split("\n")
     f.close()
 
-    global qt_dir, qt_incdir, qt_libdir, qt_bindir, qt_datadir, qt_pluginsdir
+    global qt_dir, qt_incdir, qt_libdir, qt_bindir, qt_datadir, qt_archdatadir
+    global qt_pluginsdir
     global qt_version, qt_edition, qt_licensee, qt_shared, qt_xfeatures
 
     qt_dir = lines[0]
@@ -2166,12 +2216,13 @@ int main(int argc, char **argv)
     qt_libdir = lines[2]
     qt_bindir = lines[3]
     qt_datadir = lines[4]
-    qt_pluginsdir = lines[5]
-    qt_version = lines[6]
-    qt_edition = lines[7]
-    qt_licensee = lines[8]
-    qt_shared = lines[9]
-    qt_xfeatures = lines[10:]
+    qt_archdatadir = lines[5]
+    qt_pluginsdir = lines[6]
+    qt_version = lines[7]
+    qt_edition = lines[8]
+    qt_licensee = lines[9]
+    qt_shared = lines[10]
+    qt_xfeatures = lines[11:]
 
     if opts.assume_shared:
         qt_shared = "shared"
@@ -2288,11 +2339,23 @@ def main():
     check_vendorid()
 
     # Set the SIP platform, version and feature flags.
-    set_sip_flags(pyqt)
+    sip_flags = set_sip_flags(pyqt)
 
     # Tell the user what's been found.
     inform_user()
 
+    # Embed the sip flags.
+    sipconfig.inform("Embedding sip flags...")
+    in_f = open(os.path.join(src_dir, "qpy", "QtCore", "qpycore_post_init.cpp.in"))
+    out_f = open(os.path.join(src_dir, "qpy", "QtCore", "qpycore_post_init.cpp"), "wt")
+
+    for line in in_f:
+        line = line.replace("@@PYQT_SIP_FLAGS@@", sip_flags)
+        out_f.write(line)
+
+    in_f.close()
+    out_f.close()
+
     # Generate the code.
     pyqt.code()
 
