@@ -1,33 +1,60 @@
 #!/usr/bin/env python
 
-############################################################################
+
+#############################################################################
 ##
-## Copyright (C) 2004-2005 Trolltech AS. All rights reserved.
+## Copyright (C) 2010 Riverbank Computing Limited.
+## Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+## All rights reserved.
 ##
-## This file is part of the example classes of the Qt Toolkit.
+## This file is part of the examples of PyQt.
 ##
-## This file may be used under the terms of the GNU General Public
-## License version 2.0 as published by the Free Software Foundation
-## and appearing in the file LICENSE.GPL included in the packaging of
-## this file.  Please review the following information to ensure GNU
-## General Public Licensing requirements will be met:
-## http://www.trolltech.com/products/qt/opensource.html
+## $QT_BEGIN_LICENSE:BSD$
+## You may use this file under the terms of the BSD license as follows:
 ##
-## If you are unsure which license is appropriate for your use, please
-## review the following information:
-## http://www.trolltech.com/products/qt/licensing.html or contact the
-## sales department at sales@trolltech.com.
+## "Redistribution and use in source and binary forms, with or without
+## modification, are permitted provided that the following conditions are
+## met:
+##   * Redistributions of source code must retain the above copyright
+##     notice, this list of conditions and the following disclaimer.
+##   * Redistributions in binary form must reproduce the above copyright
+##     notice, this list of conditions and the following disclaimer in
+##     the documentation and/or other materials provided with the
+##     distribution.
+##   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
+##     the names of its contributors may be used to endorse or promote
+##     products derived from this software without specific prior written
+##     permission.
 ##
-## This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-## WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+## "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+## LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+## A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+## OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+## SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+## LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+## DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+## THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+## OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
+## $QT_END_LICENSE$
 ##
-############################################################################
+#############################################################################
+
+
+# These are only needed for Python v2 but are harmless for Python v3.
+import sip
+sip.setapi('QString', 2)
+sip.setapi('QVariant', 2)
 
 import math
 
 from PyQt4 import QtCore, QtGui
 
-import chart_rc
+try:
+    import chart_rc3
+except ImportError:
+    import chart_rc2
 
 
 class PieView(QtGui.QAbstractItemView):
@@ -54,9 +81,9 @@ class PieView(QtGui.QAbstractItemView):
         for row in range(self.model().rowCount(self.rootIndex())):
 
             index = self.model().index(row, 1, self.rootIndex())
-            value, _ = self.model().data(index).toDouble()
+            value = self.model().data(index)
 
-            if value > 0.0:
+            if value is not None and value > 0.0:
                 self.totalValue += value
                 self.validItems += 1
 
@@ -97,7 +124,7 @@ class PieView(QtGui.QAbstractItemView):
             for row in range(self.model().rowCount(self.rootIndex())):
 
                 index = self.model().index(row, 1, self.rootIndex())
-                value, _ = self.model().data(index).toDouble()
+                value = self.model().data(index)
 
                 if value > 0.0:
                     sliceAngle = 360*value/self.totalValue
@@ -115,7 +142,7 @@ class PieView(QtGui.QAbstractItemView):
             for row in range(self.model().rowCount(self.rootIndex())):
 
                 index = self.model().index(row, 1, self.rootIndex())
-                if self.model().data(index).toDouble()[0] > 0.0:
+                if self.model().data(index) > 0.0:
 
                     if listItem == validRow:
                         return self.model().index(row, 0, self.rootIndex())
@@ -141,11 +168,11 @@ class PieView(QtGui.QAbstractItemView):
         else:
             valueIndex = index
 
-        if self.model().data(valueIndex).toDouble()[0] > 0.0:
+        if self.model().data(valueIndex) > 0.0:
 
             listItem = 0
             for row in range(index.row()-1, -1, -1):
-                if self.model().data(self.model().index(row, 1, self.rootIndex())).toDouble()[0] > 0.0:
+                if self.model().data(self.model().index(row, 1, self.rootIndex())) > 0.0:
                     listItem += 1
 
             if index.column() == 0:
@@ -166,14 +193,14 @@ class PieView(QtGui.QAbstractItemView):
         if index.column() != 1:
             return QtGui.QRegion(self.itemRect(index))
 
-        if self.model().data(index).toDouble()[0] <= 0.0:
+        if self.model().data(index) <= 0.0:
             return QtGui.QRegion()
 
         startAngle = 0.0
         for row in range(self.model().rowCount(self.rootIndex())):
 
             sliceIndex = self.model().index(row, 1, self.rootIndex())
-            value, _ = self.model().data(sliceIndex).toDouble()
+            value = self.model().data(sliceIndex)
 
             if value > 0.0:
                 angle = 360*value/self.totalValue
@@ -277,14 +304,14 @@ class PieView(QtGui.QAbstractItemView):
             for row in range(self.model().rowCount(self.rootIndex())):
 
                 index = self.model().index(row, 1, self.rootIndex())
-                value, _ = self.model().data(index).toDouble()
+                value = self.model().data(index)
 
                 if value > 0.0:
                     angle = 360*value/self.totalValue
 
                     colorIndex = self.model().index(row, 0, self.rootIndex())
-                    color = QtGui.QColor(self.model().data(colorIndex,
-                                   QtCore.Qt.DecorationRole).toString())
+                    color = self.model().data(colorIndex,
+                            QtCore.Qt.DecorationRole)
 
                     if self.currentIndex() == index:
                         painter.setBrush(QtGui.QBrush(color,
@@ -306,7 +333,7 @@ class PieView(QtGui.QAbstractItemView):
 
             for row in range(self.model().rowCount(self.rootIndex())):
                 index = self.model().index(row, 1, self.rootIndex())
-                value, _ = self.model().data(index).toDouble()
+                value = self.model().data(index)
 
                 if value > 0.0:
                     labelIndex = self.model().index(row, 0, self.rootIndex())
@@ -330,9 +357,9 @@ class PieView(QtGui.QAbstractItemView):
     def rowsInserted(self, parent, start, end):
         for row in range(start, end + 1):
             index = self.model().index(row, 1, self.rootIndex())
-            value, _ = self.model().data(index).toDouble()
+            value = self.model().data(index)
 
-            if value > 0.0:
+            if value is not None and value > 0.0:
                 self.totalValue += value
                 self.validItems += 1
 
@@ -341,8 +368,9 @@ class PieView(QtGui.QAbstractItemView):
     def rowsAboutToBeRemoved(self, parent, start, end):
         for row in range(start, end + 1):
             index = self.model().index(row, 1, self.rootIndex())
-            value, _ = self.model().data(index).toDouble()
-            if value > 0.0:
+            value = self.model().data(index)
+
+            if value is not None and value > 0.0:
                 self.totalValue -= value
                 self.validItems -= 1
 
@@ -446,13 +474,13 @@ class MainWindow(QtGui.QMainWindow):
     def __init__(self):
         super(MainWindow, self).__init__()
 
-        fileMenu = QtGui.QMenu(self.tr("&File"), self)
-        openAction = fileMenu.addAction(self.tr("&Open..."))
-        openAction.setShortcut(QtGui.QKeySequence(self.tr("Ctrl+O")))
-        saveAction = fileMenu.addAction(self.tr("&Save As..."))
-        saveAction.setShortcut(QtGui.QKeySequence(self.tr("Ctrl+S")))
-        quitAction = fileMenu.addAction(self.tr("E&xit"))
-        quitAction.setShortcut(QtGui.QKeySequence(self.tr("Ctrl+Q")))
+        fileMenu = QtGui.QMenu("&File", self)
+        openAction = fileMenu.addAction("&Open...")
+        openAction.setShortcut("Ctrl+O")
+        saveAction = fileMenu.addAction("&Save As...")
+        saveAction.setShortcut("Ctrl+S")
+        quitAction = fileMenu.addAction("E&xit")
+        quitAction.setShortcut("Ctrl+Q")
 
         self.setupModel()
         self.setupViews()
@@ -464,17 +492,15 @@ class MainWindow(QtGui.QMainWindow):
         self.menuBar().addMenu(fileMenu)
         self.statusBar()
 
-        self.openFile(QtCore.QString(":/Charts/qtdata.cht"))
+        self.openFile(':/Charts/qtdata.cht')
 
-        self.setWindowTitle(self.tr("Chart"))
+        self.setWindowTitle("Chart")
         self.resize(870, 550)
 
     def setupModel(self):
         self.model = QtGui.QStandardItemModel(8, 2, self)
-        self.model.setHeaderData(0, QtCore.Qt.Horizontal,
-                QtCore.QVariant(self.tr("Label")))
-        self.model.setHeaderData(1, QtCore.Qt.Horizontal,
-                QtCore.QVariant(self.tr("Quantity")))
+        self.model.setHeaderData(0, QtCore.Qt.Horizontal, "Label")
+        self.model.setHeaderData(1, QtCore.Qt.Horizontal, "Quantity")
 
     def setupViews(self):
         splitter = QtGui.QSplitter()
@@ -496,15 +522,13 @@ class MainWindow(QtGui.QMainWindow):
 
         self.setCentralWidget(splitter)
 
-    def openFile(self, path=QtCore.QString()):
-        if path.isNull():
-            fileName = QtGui.QFileDialog.getOpenFileName(self,
-                    self.tr("Choose a data file"), "", "*.cht")
-        else:
-            fileName = path
+    def openFile(self, path=None):
+        if not path:
+            path = QtGui.QFileDialog.getOpenFileName(self,
+                    "Choose a data file", '', '*.cht')
 
-        if not fileName.isEmpty():
-            f = QtCore.QFile(fileName)
+        if path:
+            f = QtCore.QFile(path)
 
             if f.open(QtCore.QFile.ReadOnly | QtCore.QFile.Text):
                 stream = QtCore.QTextStream(f)
@@ -514,49 +538,48 @@ class MainWindow(QtGui.QMainWindow):
                         QtCore.QModelIndex())
 
                 row = 0
-                while True:
+                line = stream.readLine()
+                while line:
+                    self.model.insertRows(row, 1, QtCore.QModelIndex())
+
+                    pieces = line.split(',')
+                    self.model.setData(self.model.index(row, 0, QtCore.QModelIndex()),
+                                pieces[0])
+                    self.model.setData(self.model.index(row, 1, QtCore.QModelIndex()),
+                                float(pieces[1]))
+                    self.model.setData(self.model.index(row, 0, QtCore.QModelIndex()),
+                                QtGui.QColor(pieces[2]),
+                                QtCore.Qt.DecorationRole)
+
+                    row += 1
                     line = stream.readLine()
-                    if not line.isEmpty():
-                        self.model.insertRows(row, 1, QtCore.QModelIndex())
-
-                        pieces = line.split(",", QtCore.QString.SkipEmptyParts)
-                        self.model.setData(self.model.index(row, 0, QtCore.QModelIndex()),
-                                QtCore.QVariant(pieces[0]))
-                        self.model.setData(self.model.index(row, 1, QtCore.QModelIndex()),
-                                QtCore.QVariant(pieces[1]))
-                        self.model.setData(self.model.index(row, 0, QtCore.QModelIndex()),
-                                QtCore.QVariant(QtGui.QColor(pieces[2])), QtCore.Qt.DecorationRole)
-                        row += 1
-
-                    if line.isEmpty():
-                        break
 
                 f.close()
-                self.statusBar().showMessage(self.tr("Loaded %1").arg(fileName), 2000)
+                self.statusBar().showMessage("Loaded %s" % path, 2000)
 
     def saveFile(self):
-        fileName = QtGui.QFileDialog.getSaveFileName(self,
-                self.tr("Save file as"), "", "*.cht")
+        fileName = QtGui.QFileDialog.getSaveFileName(self, "Save file as", '',
+                '*.cht')
 
-        if not fileName.isEmpty():
+        if fileName:
             f = QtCore.QFile(fileName)
 
             if f.open(QtCore.QFile.WriteOnly | QtCore.QFile.Text):
                 for row in range(self.model.rowCount(QtCore.QModelIndex())):
-                    pieces = QtCore.QStringList()
+                    pieces = []
 
                     pieces.append(self.model.data(self.model.index(row, 0, QtCore.QModelIndex()),
-                            QtCore.Qt.DisplayRole).toString())
-                    pieces.append(self.model.data(self.model.index(row, 1, QtCore.QModelIndex()),
-                            QtCore.Qt.DisplayRole).toString())
+                            QtCore.Qt.DisplayRole))
+                    pieces.append(str(self.model.data(self.model.index(row, 1, QtCore.QModelIndex()),
+                            QtCore.Qt.DisplayRole)))
                     pieces.append(self.model.data(self.model.index(row, 0, QtCore.QModelIndex()),
-                            QtCore.Qt.DecorationRole).toString())
+                            QtCore.Qt.DecorationRole).name())
 
-                    f.write(QtCore.QByteArray(str(pieces.join(","))))
-                    f.write("\n")
+                    f.write(QtCore.QByteArray(','.join(pieces)))
+                    f.write('\n')
 
             f.close()
-            self.statusBar().showMessage(self.tr("Saved %1").arg(fileName), 2000)
+            self.statusBar().showMessage("Saved %s" % fileName, 2000)
 
 
 if __name__ == '__main__':
